/*******************
by Marco Brunetta
mbrunetta@gmail.com
********************/

( --It's always a good idea to keep the whole script is inside parenthesis to ensure you don't store any variables in global scop unnecessarily
	rollout heightColorizer "Height Colorizer"
	(
		--START OF ROLLOUT CONTROLS--
		--When setting the "pos" property of the controls is often a good idea to make them relative to eachother, so that if at one point you decide to change one, the others will adjust automatically
		groupBox gbox_options "Options" width:140 height:125
		colorPicker cpck_color1 "Color 1: " width:100 color:white pos:[30,25] 
		colorPicker cpck_color2 "Color 2: " width:100 color:blue pos:(cpck_color1.pos+[-43,23]) 
		checkBox cbx_assignMaterial "Assign Material" checked:false pos:(cpck_color2.pos+[-50,30])
		materialButton mbtn_mat "EMPTY" width:125 pos:(cbx_assignMaterial.pos+[-1,20]) enabled:false
		
		groupBox gbox_settings "Settings" width:140 height:170 pos:(gbox_options.pos+[0,gbox_options.height+10])
		spinner spn_variation "Variation:" range:[0,100,50] type:#integer fieldWidth:40 pos:(gbox_settings.pos+[20,25]) 
		spinner spn_bias "Bias:" range:[-100,100,0] type:#integer fieldWidth:40 pos:(spn_variation.pos+[-63,20]) 
		
		spinner spn_offset "Offset:" range:[-50,50,0] type:#integer fieldWidth:40 pos:(spn_bias.pos+[-71,30]) 
		spinner spn_minLimit "Low Limit:" range:[0,100,0] type:#integer fieldWidth:40 pos:(spn_offset.pos+[-87,20])
		spinner spn_maxLimit "High Limit:" range:[0,100,100] type:#integer fieldWidth:40 pos:(spn_minLimit.pos+[-89,20])
		checkBox cbx_useGradient "Use Gradient" pos:(spn_maxLimit.pos+[-75,30])
		
		button btn_clearMats "Remove Materials" width:140 height:18 pos:(cbx_useGradient.pos+[-29,30])
		button btn_start "START!" width:140 height:30 pos:(btn_clearMats.pos+[0,22])
		--END OF ROLLOUT CONTROLS--
		
		ON cbx_assignMaterial CHANGED arg DO --Event for cbx_assignMaterial checkbox
		(
			--This event enables and disables the materialButton according to the checkbox status.
			--While not necesary for the working of the script, it gives the user some feedback as to which controls he need to manipulate 
			mbtn_mat.enabled = arg
		)
		
		ON mbtn_mat PICKED arg DO --Event for the material button
		(
			IF arg != undefined THEN --We do a simple check to ensure that a material was selected
			(
				--When using a material source the script is going to need to access the material property for the diffuse color,
				--However, since this property's can change depending on the shader type, we'll need to hardcode its name,
				--so in this step we'll check the shader type to keep the user from selecting an incompatible shader.
				--I've set this script to work with the standard material, vray material, the mental ray arch & design material and finalRender's architectural material
				IF (classOf arg == standardMaterial) OR (classOf arg == fR_Architectural) OR (classOf arg == VRayMtl) OR (classOf arg == Arch___Design__mi) THEN --using the classOf function we check the material type
				(
					mbtn_mat.text = arg.name --If the material type is compatible, we set the text of the materialButton to it's name
				)
				ELSE --If it isn't compatible we let the user know using a messagebox and we clear the materialButton
				(
					messageBox "This shader is not supported by the script, sorry dude" title:"ATTENTION!"
					mbtn_mat.text = "EMPTY"
					mbtn_mat.material = undefined
				)
			)
			ELSE --If no material was selected, we clear the materialButton
			(
				mbtn_mat.text = "EMPTY"
				mbtn_mat.material = undefined
			)
		)
		
		ON btn_clearMats PRESSED DO --Event for "Clear Materials" button. 
		(
			selection.material = undefined --Simply removes all materials from selected items. I added this to the script in case the user wants to use wirecolor mode after using material mode.
		)
		
		ON btn_start PRESSED DO --Event for "START!" buton, and thus where the real fun begins...
		(
			--The first step will be to gather all the required data for the operation and store it in variables:
			local theObjects = selection as array --Store the current selection in an array.
			local deltaZ = selection.max.z-selection.min.z --Store the total height of the selection, by finding the highest point and substracting the lowest.
			local minZ = selection.min.z --Store the selection's lowest point
			local maxHeightLimit = (spn_maxLimit.value as float/100.0*deltaZ)+minZ as float --This finds the height coordinate for the max height limit. The user specifies this as a percentage, but we need to find the actual value for the calculations
			local minHeightLimit = (spn_minLimit.value as float/100.0*deltaZ)+minZ as float --Same as above but for the min height limit.
			
			--We'll also create two copies of the material in case the user checks the assign material option but not the gradient option.
			--By creating these copies outside the FOR loop we keep the script from creating a new copy for each object.
			--This way we only create two materials on the scene, saving resources, and keeping it easier to modify later.
			local newMat1 = copy mbtn_mat.material 
			local newMat2 = copy mbtn_mat.material
				
			--Once all the required data has been gathered, we loop trhough the objects in the selection to set the material/wirecolor:
			FOR obj in theObjects WHERE (isProperty obj "wireColor" AND isProperty obj "material") DO --We filter out the objects that don't have a wireColor and material property to avoid errors later on
			(
				local objValue = ((obj.pos.z-minZ)*100/deltaZ)+(random (-spn_variation.value+spn_bias.value) (spn_variation.value+spn_bias.value)) --This somewhat complex line determines the "value" of the object, which in turn is used to determine the actual color
				IF cbx_useGradient.checked THEN --A simple IF statment that changes the behavior of the script according to whether or not the "Use Gradient" option is checked 
				(
					--When the "Use Gradient" options is checked, determining the final color of the object is substancialy more complicated.
					--The first step is to determine the lowest value and delta value (difference between the highest and lowest value) for the hue, saturation and value of the chosen colors
					--Once we've done this, we'll have the range of values along which the individual color components can vary. 
					
					--We start by declaring the variables we intend to use
					local minH
					local minS
					local minV
					local deltaH
					local deltaS
					local deltaV
					--Then we look for the smaller hue between the two colors to find the minimum hue value and delta value
					IF cpck_color1.color.h >= cpck_color2.color.h THEN
					(
						minH = cpck_color2.color.h
						deltaH = cpck_color1.color.h-cpck_color2.color.h
					)
					ELSE
					(
						minH = cpck_color1.color.h
						deltaH = cpck_color2.color.h-cpck_color1.color.h
					)
					--And repeat for saturation
					IF cpck_color1.color.S >= cpck_color2.color.S THEN 
					(
						minS = cpck_color2.color.s
						deltaS = cpck_color1.color.s-cpck_color2.color.s
					)
					ELSE
					(
						minS = cpck_color1.color.s
						deltaS = cpck_color2.color.s-cpck_color1.color.s
					)
					--And again for value
					--This could be simplified by creating a function and calling it three times, but being just a few lines of code I decided to keep it this way
					IF cpck_color1.color.v >= cpck_color2.color.v THEN 
					(
						minV = cpck_color2.color.v
						deltaV = cpck_color1.color.v-cpck_color2.color.v
					)
					ELSE
					(
						minV = cpck_color1.color.v
						deltaV = cpck_color2.color.v-cpck_color1.color.v
					)
					--Using the data colected, we find the value for Hue, Saturation and Value for the current object, taking into acount the user settings
					local colorH = deltaH/100*objValue+(spn_offset.value)+MinH
					local colorS = deltaS/100*objValue+(spn_offset.value)+minS
					local colorV = deltaV/100*objValue+(spn_offset.value)+minV
					--Because of the math operations the values of H, S and V can sometimes go outside the 0-255 range, so the following lines ensure they stay inside 
					IF colorH < 0 DO colorH = 0 
					IF colorS < 0 DO colorS = 0
					IF colorV < 0 DO colorV = 0
					IF colorH > 255 DO colorH = 255
					IF colorS > 255 DO colorS = 255
					IF colorV > 255 DO colorV = 255
					
					--Because we can't construct a color directly from H S and V, we need to create one, and then set it's properties using the previous results
					local finalColor = (color 255 0 0)
					finalColor.h = colorH
					finalColor.s = colorS
					finalColor.v = colorV
					
					--Once the color has been faound is simply a matter of assigning it to the objects wireColor or creating a material, according to what the user chose.
					--On this step we also check the Max Height and Min Height constrains
					
					--We first have to find if the user chose to assign the color to the wireframe of the object's material
					IF cbx_assignMaterial.checked AND mbtn_mat.material != undefined THEN
					(
						local newMat = copy mbtn_mat.material --We create a copy of the material inside the materialButton
						CASE classOf newMat OF --We check the type of material in order to apply the color to the right property using a CASE statement
						(
							--For each obj, we first check if it's outside the max and min limits defined by the user, if it's not, we assign the color we obtained previously
							--Again, this could be optimized by creating a function and calling it 4 times, but I kept it this way so it's easier to read.
							standardMaterial:
							(
								IF obj.pos.z >= maxHeightLimit THEN newMat.diffuse = cpck_color1.color ELSE
								(
									IF obj.pos.z <= minHeightLimit THEN newMat.diffuse = cpck_color2.color ELSE
									(

										newMat.diffuse = finalColor
									)
								)
								obj.material  = newMat
							)
							fR_Architectural:
							(
								IF obj.pos.z >= maxHeightLimit THEN newMat.diffuse_color = cpck_color1.color ELSE
								(
									IF obj.pos.z <= minHeightLimit THEN newMat.diffuse_color = cpck_color2.color ELSE
									(
										newMat.diffuse_color = finalColor
									)
								)
								obj.material  = newMat
							)
							VRayMtl:
							(
								IF obj.pos.z >= maxHeightLimit THEN newMat.diffuse = cpck_color1.color ELSE
								(
									IF obj.pos.z <= minHeightLimit THEN newMat.diffuse = cpck_color2.color ELSE
									(
										newMat.diffuse = finalColor
									)
								)
								obj.material  = newMat
							)
							Arch___Design__mi:
							(
								IF obj.pos.z >= maxHeightLimit THEN newMat.diff_color = cpck_color1.color ELSE
								(
									IF obj.pos.z <= minHeightLimit THEN newMat.diff_color = cpck_color2.color ELSE
									(
										newMat.diff_color = finalColor
									)
								)
								obj.material  = newMat
							)
						)
					)
					ELSE
					(
						--If the assign material option was not checked, we simply assign the color to the wirecolor property, considering the user limits.
						IF obj.pos.z >= maxHeightLimit THEN obj.wireColor = cpck_color1.color ELSE
						(
							IF obj.pos.z <= minHeightLimit THEN obj.wireColor = cpck_color2.color ELSE
							(
								obj.wireColor = finalColor
							)
						)
					)
				)
				ELSE --If the "Use Gradient" option is not checked the process is somewhat simpler and we use the two materials we defined outside the loop
				(
					IF cbx_assignMaterial.checked AND mbtn_mat.material != undefined THEN
					(
						CASE classOf mbtn_mat.material OF --Again we need to check the type of material in order to apply the color to the right property
						(
							standardMaterial:
							(
								newMat1.diffuse = cpck_color1.color
								newMat2.diffuse = cpck_color2.color
								IF obj.pos.z >= maxHeightLimit THEN obj.material = newMat1 ELSE
								(
									IF obj.pos.z <= minHeightLimit THEN obj.material = newMat2 ELSE
									(
										IF objValue >= (50+spn_offset.value) THEN obj.material = newMat1 ELSE obj.material = newMat2
									)
								)
								
							)
							fR_Architectural:
							(
								newMat1.diffuse_color = cpck_color1.color
								newMat2.diffuse_color = cpck_color2.color
								IF obj.pos.z >= maxHeightLimit THEN obj.material = newMat1 ELSE
								(
									IF obj.pos.z <= minHeightLimit THEN obj.material = newMat2 ELSE
									(
										IF objValue >= (50+spn_offset.value) THEN obj.material = newMat1 ELSE obj.material = newMat2
									)
								)
							)
							VRayMtl:
							(
								newMat1.diffuse = cpck_color1.color
								newMat2.diffuse = cpck_color2.color
								IF obj.pos.z >= maxHeightLimit THEN obj.material = newMat1 ELSE
								(
									IF obj.pos.z <= minHeightLimit THEN obj.material = newMat2 ELSE
									(
										IF objValue >= (50+spn_offset.value) THEN obj.material = newMat1 ELSE obj.material = newMat2
									)
								)
							)
							Arch___Design__mi:
							(
								newMat1.diff_color = cpck_color1.color
								newMat2.diff_color = cpck_color2.color
								IF obj.pos.z >= maxHeightLimit THEN obj.material = newMat1 ELSE
								(
									IF obj.pos.z <= minHeightLimit THEN obj.material = newMat2 ELSE
									(
										IF objValue >= (50+spn_offset.value) THEN obj.material = newMat1 ELSE obj.material = newMat2
									)
								)
							)
						)
					)
					ELSE
					(
						--With "Use Gradient" off, and no material assigning, the process is almost trvial as you can see.
						IF obj.pos.z >= maxHeightLimit THEN obj.wirecolor = cpck_color1.color ELSE
						(
							IF obj.pos.z <= minHeightLimit THEN obj.wirecolor = cpck_color2.color ELSE
							(
								IF objValue >= (50+spn_offset.value) THEN obj.wirecolor = cpck_color1.color ELSE obj.wirecolor = cpck_color2.color
							)
						)
					)
				)
			)
		)
	)
	--Finally we create the rollout... enjoy!
	createDialog heightColorizer
)